package com.test.workaround;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.el.ELException;
import javax.el.ELResolver;
import javax.el.ExpressionFactory;
import javax.el.FunctionMapper;
import javax.el.VariableMapper;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.view.facelets.ComponentHandler;
import javax.faces.view.facelets.FaceletContext;
import javax.faces.view.facelets.FaceletException;
import javax.faces.view.facelets.TagHandlerDelegate;

import com.sun.faces.facelets.FaceletContextImplBase;
import com.sun.faces.facelets.TemplateClient;
import com.sun.faces.facelets.tag.jsf.ComponentTagHandlerDelegateImpl;
import com.sun.faces.facelets.tag.jsf.TagHandlerDelegateFactoryImpl;

/**
 * Workaround for Bug https://java.net/jira/browse/JAVASERVERFACES-3508
 *
 *
 * Ensures that unique ids are generated even if not the whole jsf tree is present during
 * restore view. Later before render response newly added components caused by dynamic
 * include are discovered and handled correctly.
 * 
 *
 *
 */
public class WorkaroundTagHandlerDelegateFactoryImpl extends TagHandlerDelegateFactoryImpl {

    @Override
    public TagHandlerDelegate createComponentHandlerDelegate(final ComponentHandler owner) {
        final TagHandlerDelegate superInstance = super.createComponentHandlerDelegate(owner);
        if (!(superInstance instanceof ComponentTagHandlerDelegateImpl)) {
            return superInstance;
        }
        return new ComponentTagHandlerDelegateImpl(owner) {

            @Override
            public void apply(FaceletContext ctx, UIComponent parent) throws IOException {
                // add a local unique prefix (based on component id) to all ids to ensure that
                // even with the dynamic includes unique ids for all tags are
                // generated
                ExtraNamespaceFaceletContext enCtx = getExtraNamespaceCtx(ctx, owner, parent);
                super.apply(enCtx, parent);

            }

        };
    }

    private ExtraNamespaceFaceletContext getExtraNamespaceCtx(FaceletContext ctx, final ComponentHandler owner, UIComponent parent) {
        if (ctx instanceof ExtraNamespaceFaceletContext) {
            // reuse
            return (ExtraNamespaceFaceletContext) ctx;
        }
        String defaultTagId = ctx.generateUniqueId(owner.getTagId());
        if(parent.getId() == null) {
        	parent.getClientId();
        }
        String parentCompId = parent.getId();
        return new ExtraNamespaceFaceletContext(ctx, defaultTagId, parentCompId);
    }

    /**
     * adds parentCompId to every generated Id
     *
     */
    @SuppressWarnings("rawtypes")
    private static class ExtraNamespaceFaceletContext extends FaceletContextImplBase {

        private FaceletContextImplBase faceletContext;
        private final Map<String, Integer> ids;
        private String prefix;

        private ExtraNamespaceFaceletContext(FaceletContext faceletContext, String defaultTagId, String parentCompId) {
            this.faceletContext = (FaceletContextImplBase) faceletContext;
            this.ids = new HashMap<String, Integer>();
            if (parentCompId == null) {
                // MFR: be sure that id of component is not null in this phase. Otherwise we
                // will generate unstable ids
                throw new IllegalArgumentException("parent component id must not be null");
            }
            StringBuilder prefixBuilder = new StringBuilder(defaultTagId);
            prefixBuilder.append("_");
            prefixBuilder.append(parentCompId.hashCode());
            this.prefix = prefixBuilder.toString();
        }

        @Override
        public FacesContext getFacesContext() {
            return faceletContext.getFacesContext();
        }

        @Override
        public String generateUniqueId(String base) {
            StringBuilder uniqueIdBuilder = new StringBuilder();
            Integer cnt = ids.get(base);

            if (cnt == null) {
                ids.put(base, 0);
                uniqueIdBuilder.delete(0, uniqueIdBuilder.length());
                uniqueIdBuilder.append(prefix);
                uniqueIdBuilder.append("_");
                uniqueIdBuilder.append(base);
            } else {
                int i = cnt.intValue() + 1;
                ids.put(base, i);
                uniqueIdBuilder.delete(0, uniqueIdBuilder.length());
                uniqueIdBuilder.append(prefix);
                uniqueIdBuilder.append("_");
                uniqueIdBuilder.append(base);
                uniqueIdBuilder.append("_");
                uniqueIdBuilder.append(i);
            }

            return uniqueIdBuilder.toString();
        }

        @Override
        public ExpressionFactory getExpressionFactory() {
            return faceletContext.getExpressionFactory();
        }

        @Override
        public void setVariableMapper(VariableMapper varMapper) {
            faceletContext.setVariableMapper(varMapper);
        }

        @Override
        public void setFunctionMapper(FunctionMapper fnMapper) {
            faceletContext.setFunctionMapper(fnMapper);
        }

        @Override
        public void setAttribute(String name, Object value) {
            faceletContext.setAttribute(name, value);
        }

        @Override
        public Object getAttribute(String name) {
            return faceletContext.getAttribute(name);
        }

        @Override
        public void includeFacelet(UIComponent parent, String relativePath) throws IOException {
            faceletContext.includeFacelet(parent, relativePath);
        }

        @Override
        public void includeFacelet(UIComponent parent, URL absolutePath) throws IOException {
            faceletContext.includeFacelet(parent, absolutePath);
        }

        @Override
        public ELResolver getELResolver() {
            return faceletContext.getELResolver();
        }

        @Override
        public FunctionMapper getFunctionMapper() {
            return faceletContext.getFunctionMapper();
        }

        @Override
        public VariableMapper getVariableMapper() {
            return faceletContext.getVariableMapper();
        }

        @Override
        public Object getContext(Class key) {
            return faceletContext.getContext(key);
        }

        @Override
        public Locale getLocale() {
            return faceletContext.getLocale();
        }

        @Override
        public boolean isPropertyResolved() {
            return faceletContext.isPropertyResolved();
        }

        @Override
        public void putContext(Class key, Object contextObject) {
            faceletContext.putContext(key, contextObject);
        }

        @Override
        public void setLocale(Locale locale) {
            faceletContext.setLocale(locale);
        }

        @Override
        public void setPropertyResolved(boolean resolved) {
            faceletContext.setPropertyResolved(resolved);
        }

        @Override
        public void pushClient(TemplateClient client) {
            faceletContext.pushClient(client);
        }

        @Override
        public void popClient(TemplateClient client) {
            faceletContext.popClient(client);
        }

        @Override
        public void extendClient(TemplateClient client) {
            faceletContext.extendClient(client);
        }

        @Override
        public boolean includeDefinition(UIComponent parent, String name) throws IOException, FaceletException, FacesException, ELException {
            return faceletContext.includeDefinition(parent, name);
        }
    }

}
